#include <stdio.h>
#include <math.h>
/* calc_sencos.c : file sorgente di un programma che calcola i valori
                   delle funzioni seno e coseno di un angolo x compreso
		   tra 0 e pigrecomezzi utilizzando le espansioni
		   in serie.
                   Le espansioni in serie sono calcolate a partire da
		   un angolo del tipo (i * pigreco) / (2 * pot2fin)
		   con i compreso tra 0 e pot2fin. I valori di pot2fin
		   e del seno e del coseno dei suddetti angoli sono
		   letti dal file "calc_sencos.inp" che e' preparato
		   preventivamente dall'esecuzione del programma in
		   prepara_sencos.c */
/* ### */
/* La prossima definizione consente di cambiare facilmente il numero
   massimo di seni e coseni che possonon essere calcolati */
#define POT2MAX 1024
/* Definizione delle variabili globali */
int pot2fin;
double pigrecomezzi, vet_seni[POT2MAX+1], vet_coseni[POT2MAX+1];
/* Definizione dei prototipi delle funzioni */
double cos_mio(double x);
double sin_mio(double x);
/* ### */
int main()
{
  int i;
  double x, y;
  FILE *ifp;
  /* Definizione di pigrecomezzi (di solito lo si pone uguale a 2
     volte il valore dell'arcotangente di 1, calcolata utilizzando la
     funzione atan che fa parte della libreria matematica) */
  pigrecomezzi = 2 * atan(1.);
  /* Apriamo in modalita' di lettura il file di output che si chiama
     "calc_sencos.inp", associamo a ofp l'indirizzo corrispondente al
     file */
  ifp = fopen("calc_sencos.inp", "r");
  /* Leggiamo in modalita' binaria (coerentemente con la modalita' di
     scrittura utilizzata in prepara_sencos.c) il valore di pot2fin
     (che gioca il ruolo di N nel testo del tema d'esame) dal file
     "calc_sencos.inp" */
  fread(&pot2fin,sizeof(int),1,ifp);
  /* Eseguiamo il seguente ciclo di istruzioni mentre il contatore i
     va da 0 a pot2fin */
  for(i=0; i<=pot2fin; i++) {
    /* Leggiamo in modalita' binaria (coerentemente con la modalita' di
       scrittura utilizzata in prepara_sencos.c) i valori delle
       variabili vet_seni[i] e vet_coseni[i] */
    fread(&vet_seni[i],sizeof(double),1,ifp);
    fread(&vet_coseni[i],sizeof(double),1,ifp);
  }
  /* Chiudiamo il file che si chiama "calc_sencos.inp" */
  fclose(ifp);
  /* Fase di input dell'angolo x (compreso tra 0 e pigrecomezzi) */
  do {
    printf("Angolo x di cui vuoi calcolare seno e coseno? ");
    printf("(0 <= x <= pigreco/2)\n");
    scanf("%lf", &x);
  } while( x<0. || x>pigrecomezzi);
  /* Calcolo del seno utilizzando l'espansione in serie di potenze
     scritta nella function sin_mio */
  y = sin_mio(x);
  /* Stampa del risultato */
  printf("Valore da me calcolato di sen(x) : %20.16lf\n", y);
  /* Scriviamo l'errore assoluto finale rispetto al risultato dato dalla 
     funzione di libreria sin */
  printf("  Errore rispetto alla funzione di libreria : %le\n",
	 fabs(y-sin(x)) );
  /* Calcolo del coseno utilizzando l'espansione in serie di potenze
     scritta nella function sin_mio */
  y = cos_mio(x);
  /* Stampa del risultato */
  printf("Valore da me calcolato di cos(x) : %20.16lf\n", y);
  /* Scriviamo l'errore assoluto finale rispetto al risultato dato dalla 
     funzione di libreria cos */
  printf("  Errore rispetto alla funzione di libreria : %le\n",
	 fabs(y-cos(x)) );
  exit(0);
}
/* ### */
double cos_mio(x)
     double x;
{
  int i, isotto, isopra, segno;
  double x0, senx0, cosx0, h, denom, poth, ris, ris_old, add;
  /* Determinazione di x0 in modo tale che sia un multiplo
     intero di pigrecomezzi / pot2fin che ha la minima
     distanza da x */
  /* A questo scopo, determiniamo isotto come il massimo intero non
     maggiore di x / pigrecomezzi * pot2fin */
  isotto = floor( x / pigrecomezzi * pot2fin );
  /* Inoltre, poniamo isopra come il minimo intero non minore di x /
     pigrecomezzi * pot2fin */
  isopra = isotto + 1;
  /* Effettuiamo un test allo scopo di verificare se il valore
     assoluto della differenza tra
         (isotto * pigrecomezzi ) / pot2fin  e x
     e' minore o uguale del valore assoluto della differenza tra
         (isopra * pigrecomezzi ) / pot2fin e x ... */
  if( fabs( ( isotto * pigrecomezzi ) / pot2fin - x ) <=
      fabs( ( isopra * pigrecomezzi ) / pot2fin - x ) ) {
    /* ... allora x0 e' uguale a isotto volte pigrecomezzi / pot2fin */
    x0 = (isotto * pigrecomezzi) / pot2fin;
    /* Poniamo senx0 e cosx0 uguali a, rispettivamente, l'isotto-esimo
       elemento dei vettori vet_seni e vet_coseni */
    senx0 = vet_seni[isotto];  cosx0 = vet_coseni[isotto];
  }
  /* Altrimenti ... */
  else {
    /* ... allora x0 e' uguale a isopra volte pigrecomezzi / pot2fin */
    x0 = (isopra * pigrecomezzi) / pot2fin;
    /* Poniamo senx0 e cosx0 uguali a, rispettivamente, l'isopra-esimo
       elemento dei vettori vet_seni e vet_coseni */
    senx0 = vet_seni[isopra];  cosx0 = vet_coseni[isopra];
  }
  /* Definizioni iniziali prima di cominciare a considerare la sommatoria
     che definisce l'espansione in serie. Poniamo:
     h uguale alla differenza tra x e x0;
     il risultato ris inizialmente uguale a cosx0;
     il contatore i (che gioca il ruolo di j nel tema d'esame)  uguale a 0;
     la variabile segno deve essere sempre uguale a (-1) elevato alla i;
     la variabile poth (che contiene le potenze di h) deve essere sempre
     uguale a h elevato alla i;
     denom (che sara' sempre uguale al denominatore) uguale a 1; */
  h = x - x0;
  ris = cosx0;
  i = 0;
  segno = 1;
  poth = 1.;
  denom = 1.;
  /* Eseguiamo il ciclo di operazioni seguenti mentre e' verificata la
     condizione che scriviamo dopo la prossima parentesi } */
  do {
    /* Poniamo la variabile ris_old uguale a ris */
    ris_old = ris;
    /* Incrementiamo il contatore i */
    i++;
    /* Cambiamo il segno alla variabile segno */
    segno *= -1;
    /* Moltiplichiamo per h la variabile poth */
    poth *= h;
    /* Moltiplichiamo per i la variabile denom (cioe' il denominatore) */
    denom *= i;
    /* Poniamo add uguale al primo dei due addendi compresi nella
       parentesi tonda della formula che da l'espansione in serie
       del coseno, cosi' com'e' scritta nel tema d'esame */
    add = senx0 * poth / denom;
    /* Incrementiamo il contatore i */
    i++;
    /* Moltiplichiamo per h la variabile poth */
    poth *= h;
    /* Moltiplichiamo per i la variabile denom (cioe' il denominatore) */
    denom *= i;
    /* Sommiamo ad add il secondo dei due addendi compresi nella
       parentesi tonda della formula che da l'espansione in serie
       del coseno, cosi' com'e' scritta nel tema d'esame */
    add += cosx0 * poth / denom;
    /* Sommiamo a ris il contributo dato dal valore di segno
       moltiplicato per add */
    ris += segno * add;
    /* Eseguiamo il test di uscita dal ciclo: se il risultato (ris) e'
       diverso da quello all'iterazione precedente (cioe' ris_old)
       continuiamo ad iterare il ciclo */
  }  while( ris != ris_old );
  /* Usciamo dalla funzione restituendo il valore del risultato alla
     funzione chiamante */
  return(ris);
}
/* ### */
double sin_mio(x)
     double x;
{
  int i, isotto, isopra, segno;
  double x0, senx0, cosx0, h, denom, poth, ris, ris_old, add;
  /* Determinazione di x0 in modo tale che sia un multiplo
     intero di pigrecomezzi / pot2fin che ha la minima
     distanza da x */
  /* A questo scopo, determiniamo isotto come il massimo intero non
     maggiore di x / pigrecomezzi * pot2fin */
  isotto = floor( x / pigrecomezzi * pot2fin );
  /* Inoltre, poniamo isopra come il minimo intero non minore di x /
     pigrecomezzi * pot2fin */
  isopra = isotto + 1;
  /* Effettuiamo un test allo scopo di verificare se il valore
     assoluto della differenza tra
         (isotto * pigrecomezzi ) / pot2fin  e x
     e' minore o uguale del valore assoluto della differenza tra
         (isopra * pigrecomezzi ) / pot2fin e x ... */
  if( fabs( ( isotto * pigrecomezzi ) / pot2fin - x ) <
      fabs( ( isopra * pigrecomezzi ) / pot2fin - x ) ) {
    /* ... allora x0 e' uguale a isotto volte pigrecomezzi / pot2fin */
    x0 = (isotto * pigrecomezzi) / pot2fin;
    /* Poniamo senx0 e cosx0 uguali a, rispettivamente, l'isotto-esimo
       elemento dei vettori vet_seni e vet_coseni */
    senx0 = vet_seni[isotto];  cosx0 = vet_coseni[isotto];
  }
  /* Altrimenti ... */
  else {
    /* ... allora x0 e' uguale a isopra volte pigrecomezzi / pot2fin */
    x0 = (isopra * pigrecomezzi) / pot2fin;
    /* Poniamo senx0 e cosx0 uguali a, rispettivamente, l'isopra-esimo
       elemento dei vettori vet_seni e vet_coseni */
    senx0 = vet_seni[isopra];  cosx0 = vet_coseni[isopra];
  }
  /* Definizioni iniziali prima di cominciare a considerare la sommatoria
     che definisce l'espansione in serie. Poniamo:
     h uguale alla differenza tra x e x0;
     il risultato ris inizialmente uguale a senx0;
     il contatore i (che gioca il ruolo di j nel tema d'esame) uguale a 0;
     la variabile segno deve essere sempre uguale a (-1) elevato alla i;
     la variabile poth (che contiene le potenze di h) deve essere sempre
     uguale a h elevato alla i;
     denom uguale a 1; */
  h = x - x0;
  ris = senx0;
  i = 0;
  segno = 1;
  poth = 1.;
  denom = 1.;
  /* Eseguiamo il ciclo di operazioni seguenti mentre e' verificata la
     condizione che scriviamo dopo la prossima parentesi } */
  do {
    /* Poniamo la variabile ris_old uguale a ris */
    ris_old = ris;
    /* Incrementiamo il contatore i */
    i++;
    /* Cambiamo il segno alla variabile segno */
    segno *= -1;
    /* Moltiplichiamo per h la variabile poth */
    poth *= h;
    /* Moltiplichiamo per i la variabile denom (cioe' il denominatore) */
    denom *= i;
    /* Poniamo add uguale al primo dei due addendi compresi nella
       parentesi tonda della formula che da l'espansione in serie
       del seno, cosi' com'e' scritta nel tema d'esame */
    add = - cosx0 * poth / denom;
    /* Incrementiamo il contatore i */
    i++;
    /* Moltiplichiamo per h la variabile poth */
    poth *= h;
    /* Moltiplichiamo per i la variabile denom (cioe' il denominatore) */
    denom *= i;
    /* Sommiamo ad add il secondo dei due addendi compresi nella
       parentesi tonda della formula che da l'espansione in serie
       del seno, cosi' com'e' scritta nel tema d'esame */
    add += senx0 * poth / denom;
    /* Sommiamo a ris il contributo dato dal valore di segno
       moltiplicato per add */
    ris += segno * add;
    /* Eseguiamo il test di uscita dal ciclo: se il risultato (ris) e'
       diverso da quello all'iterazione precedente (cioe' ris_old)
       continuiamo ad iterare il ciclo */
  } while( ris != ris_old );
  /* Usciamo dalla funzione restituendo il valore del risultato alla
     funzione chiamante */
  return(ris);
}
