#include <stdio.h>
/* num_romani.c : file sorgente di un programma che determina
                  la rappresentazione in numeri romani di alcuni
		  numeri dati. */
/* ### */
/* La prossima definizione consente di cambiare facilmente il numero
   intero massimo che viene gestito dal programma. */
#define NUMMAX 4000
/* La prossima definizione consente di cambiare facilmente la lunghezza
   massima delle stringhe che costituiscono la rappresentazione romana 
   di un numero intero positivo. */
#define LUNGMAX 20
/* Definizione dei prototipi delle funzioni */
void conv_cifra_a_romano(int cifra, char ord_grand[],
			 char cinque_per_ord_grand[],
			 char ord_grand_successivo[], char num_romano[]);
void da_arabo_a_romano(int num_arabo, char num_romano[]);
/* ### */
int main()
{
  int num_arabo;
  char num_romano[LUNGMAX];
  char riga[LUNGMAX];
  FILE *ifp, *ofp;
  /* Input della variabile num_arabo */ 
  do {
    printf("Quale numero vuoi convertire nella rappresentazione romana?\n");
    printf("    (deve essere >0 e <%d)  ", NUMMAX);
    scanf("%d", &num_arabo);
  } while(num_arabo<=0 || num_arabo>=NUMMAX);
  /* Scriviamo la rappresentazione romana di num_arabo nella stringa
     num_romano */
  da_arabo_a_romano(num_arabo, num_romano);
  /* Stampiamo su video la stringa num_romano */
  printf("%s\n", num_romano);
  printf("\n\n");
  printf("Eseguiamo:\n");
  printf("  (1) la lettura di alcuni numeri da un file di input\n");
  printf("  (2) la conversione di questi numeri nella loro rappresentazione romana\n");
  printf("  (3) la scrittura di questi numeri su un file di output\n");
  /* Apriamo il file di input "num_arabi.inp" in lettura,
     associamo a ifp l'indirizzo corrispondente al file */
  ifp = fopen("num_arabi.dat", "r");
  /* Apriamo il file di output "num_romani.out" in scrittura,
     associamo a ofp l'indirizzo corrispondente al file */
  ofp = fopen("num_romani.out", "w");
  /* Leggiamo ciascuna riga del file di input e la scriviamo nella
     stringa riga.
     Questa istruzione di lettura la mettiamo all'interno di una
     condizione di esecuzione di un ciclo while(...) {...}.
     Continuiamo ad eseguire il ciclo, mentre e' verificata la
     condizione che il risultato restituito da fgets sia diverso da
     NULL */
  while( fgets(riga, LUNGMAX, ifp) != NULL) {
    /* Leggo dalla stringa riga il numero intero che vi e' contenuto
       e lo assegno come valore alla variabile num_arabo */
    sscanf(riga, "%d", &num_arabo);
    /* Scriviamo la rappresentazione romana di num_arabo nella stringa
       num_romano */
    da_arabo_a_romano(num_arabo, num_romano);
    /* Stampiamo sul file di output la stringa num_romano */
    fprintf(ofp, "%s\n", num_romano);
  }
  /* Chiudiamo il file di input */
  fclose(ifp);
  /* Chiudiamo il file di output */
  fclose(ofp);
  exit(0);
}
/* ### */
void conv_cifra_a_romano(cifra, ord_grand, cinque_per_ord_grand,
			 ord_grand_successivo, num_romano)
     int cifra;
     char ord_grand[], cinque_per_ord_grand[], ord_grand_successivo[];
     char num_romano[];
{
  int i;
  /* Effettuiamo un test per verificare se cifra e' uguale a 9 ... */
  if(cifra == 9) {
    /* ... Allora trascriviamo in coda alla stringa num_romano la
       stringa ord_grand ... */
    strcat(num_romano, ord_grand);
    /* ... Poi trascriviamo in coda alla stringa num_romano la stringa
       ord_grand_successivo */
    strcat(num_romano, ord_grand_successivo);
  }
  else {
    /* ... Altrimenti, effettuiamo un test per verificare se la
       cifra e' uguale a 4 ... */
    if(cifra == 4) {
      /* ... Allora trascriviamo in coda alla stringa num_romano la
	 stringa ord_grand ... */
      strcat(num_romano, ord_grand);
      /* ... E incrementiamo di uno il valore di cifra */
      cifra++;
    }
    /* Effettuiamo un test per verificare se cifra e' maggiore
       o uguale a 5 ... */
    if(cifra >= 5) {
      /* ... Allora trascriviamo in coda alla stringa num_romano la
	 stringa cinque_per_ord_grand ... */
      strcat(num_romano, cinque_per_ord_grand);
      /* ... E decrementiamo di cinque il valore di cifra */
      cifra-=5;
    }
    /* Eseguiamo il ciclo di istruzioni seguenti per i che va da 1 a
       cifra */
    for(i=1; i<=cifra; i++) {
      /* Trascriviamo in coda alla stringa num_romano la stringa
	 ord_grand */
      strcat(num_romano, ord_grand);
    }
  }
}
/* ### */
void da_arabo_a_romano(num_arabo, num_romano)
     int num_arabo;
     char num_romano[];
{
  int unita, decine, centinaia, migliaia;
  char ord_grand[2], cinque_per_ord_grand[2], ord_grand_successivo[2];
  /* Effettuiamo un test per verificare se il numero di cui
     vogliamo determinare la rappresentazione romana e' maggiore
     o uguale al numero massimo consentito ... */
  if(num_arabo >= NUMMAX) {
    /* ... Questa function non contempla questo caso.
       Allora stampiamo un messaggio di errore e usciamo dalla
       function */
    printf("Il numero di cui si richiede la conversione e' troppo grande!!\n");
    return;
  }
  /* Suddividiamo num_arabo in unita', decine, centinaia e migliaia */
  /* Le unita' sono uguali al resto della divisione intera "num_arabo
     diviso 10" */
  unita = num_arabo % 10;
  /* Ridefiniamo num_arabo come il quoziente della divisione intera
     "num_arabo diviso 10" */
  num_arabo /= 10;
  /* Le decine sono uguali al resto della divisione intera "num_arabo
     diviso 10" */
  decine = num_arabo % 10;
  /* Ridefiniamo num_arabo come il quoziente della divisione intera
     "num_arabo diviso 10" */
  num_arabo /= 10;
  /* Le centinaia sono uguali al resto della divisione intera "num_arabo
     diviso 10" */
  centinaia = num_arabo % 10;
  /* Le migliaia sono uguali al quoziente della divisione intera
     "num_arabo diviso 10" */
  migliaia = num_arabo / 10;
  /* Inizio della scrittura della rappresentazione romana del numero */
  /* Poniamo inizialmente la stringa num_romano uguale alla stringa
     vuota, ovvero in modo tale che contenga al primo posto
     l'identificatore NUL (cioe' 0) di fine stringa */
  num_romano[0] = 0;
  /* Consideriamo per prime le migliaia */
  /* Poniamo inizialmente la stringa ord_grand uguale a 'M'.
     Attenzione!!! Non dimentichiamo l'identificatore NUL di fine
     stringa, cioe' l'ultimo carattere di ord_grand deve essere uguale
     a 0 */
  ord_grand[0] = 'M';
  ord_grand[1] = 0;
  /* Poniamo inizialmente la stringa cinque_per_ord_grand uguale a
     uno spazio.
     Attenzione!!! Non dimentichiamo l'identificatore NUL di fine
     stringa, cioe' l'ultimo carattere di ord_grand deve essere uguale
     a 0 */
  cinque_per_ord_grand[0] = ' ';
  cinque_per_ord_grand[1] = 0;
  /* Poniamo inizialmente la stringa ord_grand_successivo uguale a
     uno spazio.
     Attenzione!!! Non dimentichiamo l'identificatore NUL di fine
     stringa, cioe' l'ultimo carattere di ord_grand deve essere uguale
     a 0 */
  ord_grand_successivo[0] = ' ';
  ord_grand_successivo[1] = 0;
  /* Trascriviamo il numero romano corrispondente alle migliaia
     in coda alla stringa num_romano */
  conv_cifra_a_romano(migliaia, ord_grand, cinque_per_ord_grand,
		      ord_grand_successivo, num_romano);
  /* Consideriamo ora le centinaia */
  /* Modifichiamo la stringa ord_grand in modo che sia uguale a 'C' */
  ord_grand[0] = 'C';
  /* Modifichiamo la stringa cinque_per_ord_grand in modo che sia
     uguale a 'D' */
  cinque_per_ord_grand[0] = 'D';
  /* Modifichiamo la stringa ord_grand_successivo in modo che sia
     uguale a 'M' */
  ord_grand_successivo[0] = 'M';
  /* Trascriviamo il numero romano corrispondente alle centinaia
     in coda alla stringa num_romano */
  conv_cifra_a_romano(centinaia, ord_grand, cinque_per_ord_grand,
		      ord_grand_successivo, num_romano);
  /* Consideriamo ora le decine */
  /* Modifichiamo la stringa ord_grand in modo che sia uguale a 'X' */
  ord_grand[0] = 'X';
  /* Modifichiamo la stringa cinque_per_ord_grand in modo che sia
     uguale a 'L' */
  cinque_per_ord_grand[0] = 'L';
  /* Modifichiamo la stringa ord_grand_successivo in modo che sia
     uguale a 'C' */
  ord_grand_successivo[0] = 'C';
  /* Trascriviamo il numero romano corrispondente alle decine
     in coda alla stringa num_romano */
  conv_cifra_a_romano(decine, ord_grand, cinque_per_ord_grand,
		      ord_grand_successivo, num_romano);
  /* Consideriamo ora le unita */
  /* Modifichiamo la stringa ord_grand in modo che sia uguale a 'I' */
  ord_grand[0] = 'I';
  /* Modifichiamo la stringa cinque_per_ord_grand in modo che sia
     uguale a 'V' */
  cinque_per_ord_grand[0] = 'V';
  /* Modifichiamo la stringa ord_grand_successivo in modo che sia
     uguale a 'X' */
  ord_grand_successivo[0] = 'X';
  /* Trascriviamo il numero romano corrispondente alle unita
     in coda alla stringa num_romano */
  conv_cifra_a_romano(unita, ord_grand, cinque_per_ord_grand,
		      ord_grand_successivo, num_romano);
}
